# -*- coding: mbcs -*-
#
# -------------------------------------------------------
# Programmed by: 	Bjørn Harald Snersrud
# As part of a master thesis at NTNU, spring -15.
# Main supervisor: Bjørn Haugen, IPM, NTNU.
# External supervisor: Martin Hasle, DeepOcean.
# -------------------------------------------------------
# Upgraded to Abaqus 6.14 by: Tryg-Arne Alnes Løkkeborg
# As part of pre-master thesis at NTNU, autumn -15
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
# Further developed by: Tryg-Arne Alnes Løkkeborg
# As part of a master thesis at NTNU, spring -16
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
#
import os
import glob
import pylab as py
import matplotlib as mp
import numpy as np
import input as vars
import shutil
import exceptions
import time

if os.path.isdir(vars.analysisType+'/'+vars.analysisName) and vars.doAnalyse==True:
	cont=False
	while cont==False: 
		ans=input('This analysis already exists, proceeding without changing the analysis name will delete the existing data. Shure you want to contiune without changing name? \r\n[y/n]: ')
		if ans=='n' or ans=='N':
			print('ASAT terminated by user.')
			quit()
		elif ans=='y' or ans=='Y':
			cont=True

path=os.getcwd()
if vars.doAnalyse==True:
	plotIter=np.linspace(0,vars.nIter-1,vars.nIter)
elif vars.doAnalyse==False:
	plotIter=vars.plotSelection

#If non-existing analysis name; building analysis folder hierarchy. 
#If existing analysis name; deleting existing data before filling in new.
if vars.doAnalyse:
	if os.path.isdir(vars.analysisType+'/'+vars.analysisName):
		shutil.rmtree(vars.analysisType+'/'+vars.analysisName)
	if os.path.isdir(vars.analysisType)!=1:
		os.mkdir(vars.analysisType)
	os.mkdir(vars.analysisType+'/'+vars.analysisName)
	os.mkdir(vars.analysisType+'/'+vars.analysisName+'/XYdata')
	os.mkdir(vars.analysisType+'/'+vars.analysisName+'/plots')
	os.mkdir(vars.analysisType+'/'+vars.analysisName+'/abaqus')
	
#---Saving input file 'input.py' for this analysis as 'analysisInput.py'---
if vars.doAnalyse:
	file=open('input.py','r')
	inputData1=open(vars.analysisType+'/'+vars.analysisName+'/plots/analysisInput.py','w')
	inputData2=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/analysisInput.py','w')
	inputData3=open(vars.analysisType+'/'+vars.analysisName+'/abaqus/analysisInput.py','w')
	data=file.read()
	inputData1.write(data)
	inputData2.write(data)
	inputData3.write(data)
	file.close()
	inputData1.close()
	inputData2.close()
	inputData3.close()
	#--------------------------------------------------------------------------------------------------
	
	
	#Running abaqus with or without GUI, depending on useGUI=True/False
	if vars.useGUI==False:
		os.system('abaqus cae nogui='+path+'/machinery.py')
	if vars.useGUI==True:
		os.system('abaqus cae script='+path+'/machinery.py')
#Allocating function handles only if plot option is checked
if vars.doPlot:
#Generating figures
	mp.rcParams.update({'font.size': 9})
	
	sepTime=vars.T[0]
	if vars.makeBefore==True:
		sepTime+=vars.Tb[0]
	
	p11=py.figure()
	ax11=p11.add_subplot(111)
	ax11.locator_params(tight=False, nbins=7)
	y_formatter = mp.ticker.EngFormatter(unit='N', places=2)
	x_formatter=mp.ticker.EngFormatter(unit='s',places=1)
	ax11.xaxis.set_major_formatter(x_formatter)
	ax11.yaxis.set_major_formatter(y_formatter)
	py.grid()
	if vars.makeBefore==True and vars.parameterName!='Tb':
		#py.vlines(x=vars.T[0], ymin=0, ymax=5E6, linestyles='dashed')
		py.axvline(x=vars.Tb[0], color='r', ls='--', lw=0.5,) #Best alternative so far, possible to insert label='Step1->2'
	if vars.makeAfter==True and vars.parameterName!='Tb' and vars.parameterName!='T':
		py.axvline(x=sepTime, color='r', ls='--', lw=0.5)
	py.title('Wire force at winch')
	py.xlabel('Analysis time')
	py.ylabel('Force')
	
	p12=py.figure()
	ax12=p12.add_subplot(111)
	ax12.locator_params(tight=False, nbins=7)
	ax12.xaxis.set_major_formatter(x_formatter)
	ax12.yaxis.set_major_formatter(y_formatter)
	py.grid()
	if vars.makeBefore==True and vars.parameterName!='Tb':
		py.axvline(x=vars.Tb[0], color='r', linestyle='--', lw=0.5)
	if vars.makeAfter==True and vars.parameterName!='Tb' and vars.parameterName!='T':
		py.axvline(x=sepTime, color='r', linestyle='--', lw=0.5)
	py.title('Wire force mid-fjord')
	py.xlabel('Analysis time')
	py.ylabel('Force')
	
	p2=py.figure()
	py.grid()
	py.title('Mid-fjord bundle displacement')
	py.xlabel('Analysis time')
	py.ylabel('Displacement')
	if vars.makeBefore==True and vars.parameterName!='Tb':
		py.axvline(x=vars.Tb[0], color='r', linestyle='--', lw=0.5)
	if vars.makeAfter==True and vars.parameterName!='Tb' and vars.parameterName!='T':
		py.axvline(x=sepTime, color='r', linestyle='--', lw=0.5)
	ax2=p2.add_subplot(111)
	ax2.locator_params(tight=False, nbins=7)
	y_formatter = mp.ticker.EngFormatter(unit='m', places=2)
	x_formatter=mp.ticker.EngFormatter(unit='s',places=1)
	ax2.xaxis.set_major_formatter(x_formatter)
	ax2.yaxis.set_major_formatter(y_formatter)
	
	p41=py.figure()
	ax41=p41.add_subplot(111)
	ax41.locator_params(tight=False, nbins=7)
	py.grid()
	py.title('Stress in bundle')
	py.xlabel('Distance from shore')
	py.ylabel('Stress')
	y_formatter = mp.ticker.EngFormatter(unit='Pa', places=0)
	x_formatter=mp.ticker.EngFormatter(unit='m',places=0)
	ax41.xaxis.set_major_formatter(x_formatter)
	ax41.yaxis.set_major_formatter(y_formatter)
	
	p42=py.figure()
	ax42=p42.add_subplot(111)
	ax42.locator_params(tight=False, nbins=7)
	py.grid()
	py.title('Bundle displacement')
	py.xlabel('Distance from shore')
	py.ylabel('Displacement')
	y_formatter = mp.ticker.EngFormatter(unit='m', places=1)
	x_formatter=mp.ticker.EngFormatter(unit='m',places=1)
	ax42.xaxis.set_major_formatter(x_formatter)
	ax42.yaxis.set_major_formatter(y_formatter)
	
	p5=py.figure()
	ax5=p5.add_subplot(111)
	ax5.locator_params(tight=False, nbins=7)
	py.grid()
	py.title('Axial force in bundle')
	py.xlabel('Distance from shore')
	py.ylabel('Force')
	y_formatter = mp.ticker.EngFormatter(unit='N', places=2)
	x_formatter=mp.ticker.EngFormatter(unit='m',places=1)
	ax5.xaxis.set_major_formatter(x_formatter)
	ax5.yaxis.set_major_formatter(y_formatter)
	
	p6=py.figure()
	ax6=p6.add_subplot(111)
	ax6.locator_params(tight=False, nbins=7)
	py.grid()
	py.title('Relative axial force distribution in bundle')
	py.xlabel('Distance from shore')
	py.ylabel('Relative force level')
	y_formatter = mp.ticker.EngFormatter( unit='',places=2)
	x_formatter=mp.ticker.EngFormatter(unit='m',places=1)
	
	#If Fb or Fa is the varied parameter, 1 and 0 are replaced with True and False. Used for the legend in the plots
	parameterValues=vars.parameterValues
	if vars.parameterName=='Fb' or vars.parameterName=='Fa':
		tempParameterValues=list()
		for i in range(len(parameterValues)):
			if parameterValues[i]==1:
				tempParameterValues.append('True')
			elif parameterValues[i]==0:
				tempParameterValues.append('False')
			else:
				tempParameterValues.append(parameterValues[i])
		parameterValues=tempParameterValues
	
	markers='<','>','D','D','<','>','<','>','D','D','<','>','<','>','D','D','<','>','<','>','D','D','<','>','<','>','D','D','<','>','<','>','D','D','<','>'
	#Starting loop for plotting etc.
	for it in plotIter:
	#Reading data
			it=int(it)
			f11=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/CTF1_start_'+str(it)+'.txt','r')
			f12=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/CTF1_mid_'+str(it)+'.txt','r')
			f21=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/U2_midPoint_'+str(it)+'.txt','r')
			f41=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/S11_path_'+str(it)+'.txt','r')
			f42=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/U2_path_'+str(it)+'.txt','r')
			
			#The below block creates xy-data for the axial force in the bundle as a spatial variable.
			elData=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/nElements_'+str(it)+'.txt','r')
			nElements=np.loadtxt(elData,dtype='int')
			if os.path.isfile(vars.analysisType+'/'+vars.analysisName+'/XYdata/axialForceInBundle_'+str(it)+'.txt')!=True:
				f5=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/axialForceInBundle_'+str(it)+'.txt','w')
				for elNumber in np.linspace(1,nElements,nElements):
					f5_pre=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/NFORC_'+str(int(elNumber))+'_'+str(it)+'.txt','r')
					tooMuchData=np.loadtxt(f5_pre,dtype="float",skiprows=3,usecols=[0,1])
					sizes=tooMuchData.shape
					L=sizes[0]
					f5.write(str(tooMuchData[L-1][1])+'\t'+str((float(elNumber)/nElements)*3604)+'\r\n')
					f5_pre.close()
					os.remove(vars.analysisType+'/'+vars.analysisName+'/XYdata/NFORC_'+str(int(elNumber))+'_'+str(it)+'.txt')
				f5.close()
			f5=open(vars.analysisType+'/'+vars.analysisName+'/XYdata/axialForceInBundle_'+str(it)+'.txt','r')
			xy11=np.loadtxt(f11,dtype="float",skiprows=3,usecols=[0,1])
			xy12=np.loadtxt(f12,dtype="float",skiprows=3,usecols=[0,1])
			xy21=np.loadtxt(f21,dtype="float",skiprows=3,usecols=[0,1])
			xy41=np.loadtxt(f41,dtype="float",skiprows=3,usecols=[0,1])
			xy42=np.loadtxt(f42,dtype="float",skiprows=3,usecols=[0,1])
			xy5=np.loadtxt(f5,dtype='float',skiprows=0,usecols=[0,1])
			y6=xy5[:,0]
			y6=y6/max(y6)			
			#Plotting data, with label if nIterations>1, without if not
			if vars.parameterName=='none_varied':
				ax11.plot(xy11[:,0],xy11[:,1],lw=0.5,ms=3)
				ax12.plot(xy12[:,0],xy12[:,1],lw=0.5,ms=3)
				ax2.plot(xy21[:,0],-xy21[:,1],lw=0.5,ms=3)
				ax41.plot(xy41[:,0],xy41[:,1],lw=0.5,ms=3)
				ax42.plot(xy42[:,0],-xy42[:,1],lw=0.5,ms=3)
				ax5.plot(xy5[:,1],xy5[:,0],lw=0.5,ms=3)
				ax6.plot(xy5[:,1],y6,lw=0.5,ms=3)
			else:
				ax11.plot(xy11[:,0],xy11[:,1],label=vars.parameterName+' = '+str(parameterValues[it]),marker=markers[it],markevery=int(vars.markEvery[it]),lw=0.5,ms=3)
				ax12.plot(xy12[:,0],xy12[:,1],label=vars.parameterName+' = '+str(parameterValues[it]),marker=markers[it],markevery=int(vars.markEvery[it]),lw=0.5,ms=3)
				ax2.plot(xy21[:,0],-xy21[:,1],label=vars.parameterName+' = '+str(parameterValues[it]),marker=markers[it],markevery=int(vars.markEvery[it]),lw=0.5,ms=3)
				ax41.plot(xy41[:,0],xy41[:,1],label=vars.parameterName+' = '+str(parameterValues[it]),marker=markers[it],markevery=int(float(nElements)/(2.0+0.3*it)),lw=0.5,ms=3)
				ax42.plot(xy42[:,0],-xy42[:,1],label=vars.parameterName+' = '+str(parameterValues[it]),marker=markers[it],markevery=int(float(nElements)/(1.0+0.5*it)),lw=0.5,ms=3)
				ax5.plot(xy5[:,1],xy5[:,0],label=vars.parameterName+' = '+str(parameterValues[it]),marker=markers[it],markevery=int(float(nElements)/(2.0+0.5*it)),lw=0.5,ms=3)
				ax6.plot(xy5[:,1],y6,label=vars.parameterName+' = '+str(parameterValues[it]),marker=markers[it],markevery=int(float(nElements)/(2.0+0.5*it)),lw=0.5,ms=3)
	
	#Closing files
	f11.close()
	f12.close()
	f21.close()
	f41.close()
	f42.close()
	f5.close()	
	elData.close()
	
	#Setting views and dimensions to fit in a b5 thesis.
	positionVector=[0.2,0.2,0.53,0.6]
	positionVector_none_varied=[0.2,0.2,0.7,0.6]
	height=2.5
	width=5.21
	
	#Setting legend positions only if there are parameter variations in the relevant data
	if vars.parameterName!='none_varied':
		ax11.legend(loc='center left' ,prop={'size':9},frameon=False, bbox_to_anchor=(1.0,0.5))
		ax12.legend(loc='center left' ,prop={'size':9},frameon=False, bbox_to_anchor=(1.0,0.5))
		ax2.legend(loc='center left' ,prop={'size':9},frameon=False, bbox_to_anchor=(1.0,0.5))
		ax41.legend(loc='center left' ,prop={'size':9},frameon=False, bbox_to_anchor=(1.0,0.5))
		ax42.legend(loc='center left' ,prop={'size':9},frameon=False, bbox_to_anchor=(1.0,0.5))
		ax5.legend(loc='center left' ,prop={'size':9},frameon=False, bbox_to_anchor=(1.0,0.5))
		ax6.legend(loc='center left' ,prop={'size':9},frameon=False, bbox_to_anchor=(1.0,0.5))
		
		ax11.set_position(positionVector)
		ax12.set_position(positionVector)
		ax2.set_position(positionVector)

		ax41.set_position(positionVector)
		ax42.set_position(positionVector)
		ax5.set_position(positionVector)
		ax6.set_position(positionVector)
	elif vars.parameterName=='none_varied':

		ax12.set_position(positionVector_none_varied)

		ax2.set_position(positionVector_none_varied)

		ax41.set_position(positionVector_none_varied)
		ax42.set_position(positionVector_none_varied)
		ax5.set_position(positionVector_none_varied)
		ax6.set_position(positionVector_none_varied)
	#
	#
	#
	p11.set_size_inches(width,height)
	p12.set_size_inches(width,height)
	p2.set_size_inches(width,height)

	p41.set_size_inches(width,height)
	p42.set_size_inches(width,height)
	p5.set_size_inches(width,height)
	p6.set_size_inches(width,height)
	#
	#
	#
	
	#Saving plots
	
	p11.savefig(vars.analysisType+'/'+vars.analysisName+'/plots/CTF1first.pdf',dpi=300)
	p12.savefig(vars.analysisType+'/'+vars.analysisName+'/plots/CTF1mid.pdf',dpi=300)
	p2.savefig(vars.analysisType+'/'+vars.analysisName+'/plots/U2_center.pdf',dpi=300)

	p41.savefig(vars.analysisType+'/'+vars.analysisName+'/plots/path_MISES.pdf',dpi=300)
	p42.savefig(vars.analysisType+'/'+vars.analysisName+'/plots/path_U2.pdf',dpi=300)
	p5.savefig(vars.analysisType+'/'+vars.analysisName+'/plots/axialForceBundle.pdf',dpi=300)
	p6.savefig(vars.analysisType+'/'+vars.analysisName+'/plots/axialForceBundle_relative.pdf',dpi=300)
	
	#Displaying plots depending on input.
	if vars.doDisplayPlotsWhenDone:
		py.show()
#