# -*- coding: mbcs -*-
#
# -------------------------------------------------------
# Programmed by: 	Bjørn Harald Snersrud
# As part of a master thesis at NTNU, spring -15.
# Main supervisor: Bjørn Haugen, IPM, NTNU.
# External supervisor: Martin Hasle, DeepOcean.
#--------------------------------------------------------
# Upgraded to Abaqus 6.14 by: Tryg-Arne Alnes Løkkeborg
# As part of pre-master thesis at NTNU, autumn -15
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
# Further developed by: Tryg-Arne Alnes Løkkeborg
# As part of a master thesis at NTNU, spring -16
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
#
#The below code are made to inform why an analyis will not work, and to kill the process before it crashes by itself later on.
import input as vars
import numpy as np
import os

if vars.doRaiseErrors==True:
	if vars.doAnalyse:
		
		if vars.nVaried>vars.nIter-1:
			print('ASAT terminated due to parameter input error.')
			print('Too many parameters varied. Only one allowed')
			quit()
		
		for i in vars.nP:
			if int(i) % 2 != 1:
				print('ASAT terminated due to parameter input error.')
				print('There is one ore more even number(s) in nP, only odd numbers supported. Have you given odd values to nP_min and nP_max? Have you chosen an nIterations that will give all odd values through all iterations?')
				quit()
		
		if vars.nVaried==0 and vars.nIter>1:
			print('ASAT terminated due to parameter assignment missmatch')
			print('nIterations is larger than one, while no parameter is set to vary')
			quit()
		if vars.nIter==1:
			if (vars.nP_max!=vars.nP_min or vars.dL_max!=vars.dL_min or
					vars.T_max!=vars.T_min or vars.D_max!=vars.D_min or
					vars.t_max!=vars.t_min or vars.Mu_min!=vars.Mu_max or
					vars.Le_min!=vars.Le_max or vars.Lw_min!=vars.Lw_max or
					vars.Lquart_max!=vars.Lquart_min or
					vars.S_max!=vars.S_min or vars.Lmid_max!=vars.Lmid_min or
					vars.Tb_max!=vars.Tb_min or vars.Ta_max!=vars.Ta_min or
					vars.Fb_max!=vars.Fb_min or vars.Fa_max!=vars.Fa_min or
					vars.EA_max!=vars.EA_min):
				print('ASAT terminated due to parameter assignment missmatch')
				print('At least one parameter is set to vary, while nIterations is not larger than one.')
				quit()
		for j in range(vars.nIter):
			cutParameters=vars.cutAt(vars.nP[j],vars.Lmid[j],vars.Lquart[j],vars.cutMethod)
			for i in cutParameters:
				if i>=1:
					print('The pulley density function parameters creates a function value f>=1. Please reselect values Lmid, Lquart or both')
					quit()
				elif i<=0:
					print('The pulley density parameters creates a function value f<=1. Please reselect values of Lmid, Lquart or both')
					quit()
		if vars.cutMethod!='linear' and vars.cutMethod!='quadratic' and vars.cutMethod!='quartic':
			print("cutMethod='"+str(vars.cutMethod)+"' is not allowed. Proper values includes 'linear', 'quadratic' and 'quartic'.")
			quit()
		if (vars.parameterName=='Fb' or vars.parameterName=='Fa') and vars.nIter>2:
			print('ASAT terminated due to parameter missmatch')
			print(vars.parameterName+' is a True/False statement. Only two iterations are necessary')
			quit()
		if (vars.Fb_max!=True and vars.Fb_max!=False or
				vars.Fb_min!=True and vars.Fb_min!=False or
				vars.Fa_max!=True and vars.Fa_max!=False or
				vars.Fa_min!=True and vars.Fa_min!=False):
			print('ASAT terminated due to parameter assignment missmatch')
			print('Fb and Fa are True/False statements. Other values are not allowed')
			quit()
		if vars.parameterName=='Fa' and vars.Fb_max==True and vars.Fb_min==True:
			print('ASAT terminated due to parameter input error')
			print('Fb is True for all analyses, so Fa is ignored')
			print('Set Fb equal False if you want to use Fa as varied parameter')
			quit()
		if vars.wire!='rigid' and vars.wire!='elastic':
			print('ASAT terminated due to parameter assignment missmatch')
			print("wire='"+str(vars.wire)+"' is not allowed. Proper values includes 'rigid' and 'elastic'")
			quit()
	elif vars.doAnalyse==False:
		if vars.nIter<=np.amax(vars.plotSelection):
			print('ASAT terminated due to plot input range error.')
			print('The highest integer in "plotSelection" exceeds the amount of iterations in the output database.')
			print('The amount of iterations available for plotting is: '+str(vars.nIter))
			print('This makes the maximal value for plotSelection = '+str(np.linspace(0,vars.nIter-1,vars.nIter)))
			quit()

		if len(vars.plotSelection)>vars.nIter:
			print('ASAT terminated due to plot range error')
			print('The number of elements in "plotSelection" is higher than the amount of iterations in the output database.')
			print('The amount of iterations available for plotting is: '+str(vars.nIter))
			print('This makes the maximal value for plotSelection = '+str(np.linspace(0,vars.nIter-1,vars.nIter)))
			quit()

elif vars.doRaiseErrors==False:
	cont=False
	while cont !=True:
		c=input('The value of doRaiseErrors=False. ASAT will not control the analysis input, and could consequently crash without warning. Continue? \r\n[y/n]: ')
		if c=='n' or c=='N':
			print('ASAT terminated by user.')
			quit()
		elif c=='y' or c=='Y':
			cont=True
