# -*- coding: mbcs -*-
#
# -------------------------------------------------------
# Programmed by: 	Bjørn Harald Snersrud
# As part of a master thesis at NTNU, spring -15.
# Main supervisor: Bjørn Haugen, IPM, NTNU.
# External supervisor: Martin Hasle, DeepOcean.
# -------------------------------------------------------
# Upgraded to Abaqus 6.14 by: Tryg-Arne Alnes Løkkeborg
# As part of pre-master thesis at NTNU, autumn -15
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
# Further developed by: Tryg-Arne Alnes Løkkeborg
# As part of a master thesis at NTNU, spring -16
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
#
#
import numpy as np
import imp
import sys
import os
#----------------------------------------------------------------------------------------------------------------
#-----------------------------------------Precedureal user input.------------------------------------------------
#----------------------------------------------------------------------------------------------------------------
#
# 
analysisType='foo'						#Generally not varied between analyses
analysisName='foo'						#Analysis specific folder. Found inside "analysisType"-folder. Contains the analysis data hierarchy								
doAnalyse=True							#Decides whether a model shall be built and analysed or not.
doPlot=True								#Can be done at later time, if plot=True, and the two analysis options is False, this becomes a pure plotting tool. Then Remember to assign a selection of numbers to plotSelection! 
doDisplayPlotsWhenDone=True				#Will make all plots pop open at analysis end if True.
useGUI=False							#Set True if abaqus GUI wanted. This will make the analyses slower.
plotSelection=[0,1]				 		#If doAnalyse==False, this decides the iterations to plot. Proper assignment: plotSelection=[0,1,2,5,19], if nIterations was >=20
nIterations=2							#Number of parameter iterations. The analysis will run through N full cycles for nIterations=N.
doLockAll=False
#
#
#
##---------------------------------------------------------------------------------------------
#-----------------------------------FE-Analysis user input-------------------------------------
#----------------------------------------------------------------------------------------------
#																							  #
#																							  #
#																							  #
Tb_min=150 				#Analysis time for the step before the winching procedure			  #
Tb_max=150 																					  #
#																							  #
T_min=28 				#Analysis time for winching step									  #
T_max=28 																					  #
#																							  #
Ta_min=44				#Analysis time for the step after the winching procedure			  #
Ta_max=44																					  #
#																							  #
t_min=5.95				#Winching duration													  #
t_max=5.95																					  #
#																							  #
D_min=0.10				#Mass damping factor												  #
D_max=0.10																					  #
#																							  #
Lw_min=100				#Distance between abutments and winches								  #
Lw_max=100																					  #
#																							  #
cutMethod='quadratic'  	#Takes values 'linear', 'quadratic' and 'quartic'.					  #
#																							  #
Lmid_min=0.25			#Weighting of distance between two pulleys at mid-fjord				  #
Lmid_max=0.25																				  #
#																							  #
Lquart_min=0.40			#Weighting of distance beteen two pulleys at the quarter point		  #
Lquart_max=0.40																				  #
#																							  #
nP_min=13				#Amount of pulleys along each bundle								  #
nP_max=13																					  #
#																							  #
Mu_min=0.10				#Pulley friction coefficient										  #
Mu_max=0.10																					  #
#																							  #
dL_min=120				#Length of wire winched out of each of the two wire-spans.		      #
dL_max=120																					  #
#																							  #
Le_min=10 				#Element lengths													  #
Le_max=10																					  #
#																							  #
S_min=0.0				#Spacing between the three pipes in cross section					  #
S_max=0.0																					  #
#																							  #
Fb_min=True 			#Apply the load before the winching procedure. True/False statement	  #
Fb_max=False 																				  #
#																							  #
Fa_min=False  			#Apply load after winching procedure. True/False statement		 	  #
Fa_max=False 			#Argument ignored if Fb=True(load already applied)					  #
#																							  #
wire='rigid'			#Takes values 'rigid' and 'elastic'									  #
#																							  #
EA_min=100000000		#EA for the elastic wire											  #
EA_max=100000000																			  #
#																							  #
#																							  #



#---------------------------The below code is NOT to be edited by user!----------------------
if doAnalyse==False:
	if os.path.isdir(analysisType+'/'+analysisName)==False:
		print('No results found at given path. Set "doAnalyse=True" to create results, or change path variables "analysisType" and/or "analysisName" to point at existing results.')
		quit()
	sys.path.append(str(analysisType)+'/'+str(analysisName)+'/plots')
	import analysisInput as tempVars
	parameterValues=tempVars.parameterValues
	parameterName=tempVars.parameterName
	nIter=tempVars.nIterations
elif doAnalyse==True:
	nIter=nIterations	
#--------------------------------------------------------------------------------------------------------------------------------
T=np.linspace(T_min,T_max,nIter)
t=np.linspace(t_min,t_max,nIter)
D=np.linspace(D_min,D_max,nIter)
Lw=np.linspace(Lw_min,Lw_max,nIter)
Lmid=np.linspace(Lmid_min,Lmid_max,nIter)
Lquart=np.linspace(Lquart_min,Lquart_max,nIter)
nP=np.linspace(nP_min,nP_max,nIter)
Mu=np.linspace(Mu_min,Mu_max,nIter)
dL=np.linspace(dL_min,dL_max,nIter)
Le=np.linspace(Le_min,Le_max,nIter)
S=np.linspace(S_min,S_max,nIter)
Tb=np.linspace(Tb_min,Tb_max,nIter)
Ta=np.linspace(Ta_min,Ta_max,nIter)
Fb=np.linspace(Fb_min,Fb_max,nIter)
Fa=np.linspace(Fa_min,Fa_max,nIter)
EA=np.linspace(EA_min,EA_max,nIter)


#
#-----------------------Function definitons and hardcoded values---------------------------
#
def cutAt(partitions,weight_mid,weight_quart,method):
	#Weights not considered if method==constant
	weight_start=1
	parameter=list()
	n=partitions
	#
	if method=='constant':
		for i in range(partitions):
			parameter.append(1.0-1.0/(n+1))
			n=n-1
	#
	#
	if method=='linear':
		nHalf=int((partitions+1)/2)
		a=(weight_start-weight_mid)/(nHalf-1)
		weights=np.zeros(shape=(1,partitions))
		L=1
		weightSum=0
		for i in range(int(partitions)):
				if i<=nHalf-1:
					print('first')
					weights[0,i]=weight_start-a*i
				if i>nHalf-1:
					print('second')
					weights[0,i]=weight_mid+a*(i-nHalf)
				weightSum=weightSum+weights[0,i]
		weightSum=weightSum+weights[0,0]
		relativeWeights=weights/weightSum
		print(weightSum)
		print(weights)
		print(relativeWeights)
		for i in range(int(partitions)):
			parameter.append(1-(relativeWeights[0,i]/L))
			L=L-relativeWeights[0,i]
	
	if method=='quadratic':
		nHalf=int((partitions+1)/2)
		a=((2/float(partitions))**2)*(weight_start-weight_mid)
		b=-2*a*(float(partitions)/2)
		c=weight_start
		func = lambda x: a*x**2+b*x+c  
		weights=np.zeros(shape=(1,partitions))
		L=1
		weightSum=0
		for i in range(int(partitions)):
				weights[0,i]=func(i)
				weightSum=weightSum+weights[0,i]
		weightSum=weightSum+weights[0,0]
		relativeWeights=weights/weightSum
		for i in range(int(partitions)):
			parameter.append(1-(relativeWeights[0,i]/L))
			L=L-relativeWeights[0,i]
	if method=='quartic':
		Wk=weight_quart
		Wm=weight_mid
		nHalf=int((partitions+1)/2)
		a=64*Wm - (256*Wk)/3 + 64/3
		b=(512*Wk)/3 - 128*Wm - 128/3
		c=76*Wm - (320*Wk)/3 + 92/3
		d=(64*Wk)/3 - 12*Wm - 28/3
		e=1
		func = lambda x: a*x**4+b*x**3+c*x**2+d*x+e  
		weights=np.zeros(shape=(1,partitions))
		L=1
		weightSum=0
		for i in range(int(partitions)):
				weights[0,i]=func(float(i)/float(partitions))
				weightSum=weightSum+weights[0,i]
		weightSum=weightSum+weights[0,0]
		relativeWeights=weights/weightSum
		for i in range(int(partitions)):
			parameter.append(1-(relativeWeights[0,i]/L))
			L=L-relativeWeights[0,i]
			
	return parameter
	
parameterNames='T','t','D','Lw','nP','$\mu$','dL','Le','Lmid','Lquart', 'S', 'Tb', 'Ta', 'Fb', 'Fa', 'EA'
nVariables=len(parameterNames)
temp=np.zeros(shape=(nVariables,nIter))
temp[0,0:nIter]=T
temp[1,0:nIter]=t
temp[2,0:nIter]=D
temp[3,0:nIter]=Lw
temp[4,0:nIter]=nP
temp[5,0:nIter]=Mu
temp[6,0:nIter]=dL
temp[7,0:nIter]=Le
temp[8,0:nIter]=Lmid
temp[9,0:nIter]=Lquart
temp[10,0:nIter]=S
temp[11,0:nIter]=Tb
temp[12,0:nIter]=Ta
temp[13,0:nIter]=Fb
temp[14,0:nIter]=Fa
temp[15,0:nIter]=EA
#
nVaried=0
if nIter>1:
	parameterValues=np.zeros(shape=(1,nIter))
elif nIter==1:
	parameterName='none_varied'
	parameterValues=[0]
for i in range(nVariables):
		for j in range(nIter-1):
			if temp[i,j]!=temp[i,j+1]:
				nVaried+=1
				parameterValues=temp[i,:]
				parameterName=parameterNames[i]
if nVaried==0 and nIter>1:
	parameterName='none_varied'
#
#Decides wether or not to create a step before and/or after the winching procedure
#Creating a step before winching if Fb==True for one or more of the iterations
makeBefore=False
if Fb[0]==True or parameterName=='Fb':
	makeBefore=True
#
#Creating a step after winching if Fb or Fa==True for one or more of the iterations
makeAfter=False
if Fb[0]==True or parameterName=='Fb' or Fa[0]==True or parameterName=='Fa':
	makeAfter=True
#
#The following defines the data sampling frequencies and plotting options.
requestedFrames=20
nHistoryOutputPoints=500
Ttot=T
nSteps = 1
if makeBefore==True:
	Ttot = Ttot + Tb
	nSteps += 1
if makeAfter==True:
	Ttot = Ttot + Ta
	nSteps += 1
extractDataAtTimes=T/nHistoryOutputPoints
nMarkersMax=4			#Gives a total amount of markers per line in the plotter
markEvery=np.zeros(shape=(len(Ttot),1))
for i in range(len(Ttot)):
	markEvery[i]=(nHistoryOutputPoints*nSteps/nMarkersMax)*(max(Ttot)/Ttot[i])
#
#If plotter is used with a previously analysed model(with doAnalyse==False),
#label values are gathered from original input.py - file(renamed 'analysisInpyt.py'), stored in the respective folders.
#
if doAnalyse==False:
	sys.path.append(str(analysisType)+'/'+str(analysisName)+'/plots')
	import analysisInput as tempVars
	parameterValues=tempVars.parameterValues
	parameterName=tempVars.parameterName
	nIter=tempVars.nIter
				
				
doRaiseErrors=True #This can be checked False is one needs to perform certain tasks which requires ASAT to run through the standard termination criteria. 
