# -*- coding: mbcs -*-
#
# -------------------------------------------------------
# Programmed by: 	Bjørn Harald Snersrud
# As part of a master thesis at NTNU, spring -15.
# Main supervisor: Bjørn Haugen, IPM, NTNU.
# External supervisor: Martin Hasle, DeepOcean.
# -------------------------------------------------------
# Upgraded to Abaqus 6.14 by: Tryg-Arne Alnes Løkkeborg
# As part of pre-master thesis at NTNU, autumn -15
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
# Further developed by: Tryg-Arne Alnes Løkkeborg
# As part of a master thesis at NTNU, spring -16
# Main supervisor: Bjørn Haugen, IPM, NTNU
# -------------------------------------------------------
#
# This script is heavily dependent of Abaqus' python 
# API. For more information on predefined functions and classes, visit:
# http://www.maths.cam.ac.uk/computing/software/abaqus_docs/docs/v6.12/books/ker/default.htm
# click "Python commands" and use the search function.
# -------------------------------------------------------
#
from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import input as vars
import numpy as np
import time as timer
#
import os
os.chdir(vars.analysisType+'/'+vars.analysisName+'/abaqus')
#
#
#
#
count=0#Dummy-iteration counter.
if vars.doAnalyse==True:
	if vars.useGUI==False:
		file=open('timeData.txt','w')	#Logging times spent on each analysis.	
	startTimeTotal=timer.time()
	for it in np.linspace(0,vars.nIter-1,vars.nIter):
		startTime=timer.time()
		executeOnCaeStartup()
		Mdb()
		#
		#All constants and functions are defined in "input.py" and used as "vars.<'constant'/function(<arguments>)'>"
		#
		#
		#
		#
		#
		#---------------------------------------------------------------------------------
		#----------------------------------PART MODULE------------------------------------
		#---------------------------------------------------------------------------------
		s = mdb.models['Model-1'].ConstrainedSketch(name='__profile1__', 
			sheetSize=4000.0)
		g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
		s.setPrimaryObject(option=STANDALONE)
		s.Spot(point=(0.0, 0.0))
		s.FixedConstraint(entity=v[0])
		s.Spot(point=(3604, 0.0))
		s.FixedConstraint(entity=v[1])
		s.Arc3Points(point1=(0.0, 0.0), point2=(3604.0, 0.0), point3=(1802.0, -275.0))
		s.VerticalDimension(vertex1=v[0], vertex2=v[4], textPoint=(-595.455078125, 
			-131.898254394531), value=360)
		s.HorizontalDimension(vertex1=v[4], vertex2=v[1], textPoint=(2994.951171875, 
			740.461364746094), value=1802.0)
		p = mdb.models['Model-1'].Part(name='as1', dimensionality=THREE_D, 
			type=DEFORMABLE_BODY)
		p = mdb.models['Model-1'].parts['as1']
		p.BaseWire(sketch=s)
		s.unsetPrimaryObject()
		p = mdb.models['Model-1'].parts['as1']
		del mdb.models['Model-1'].sketches['__profile1__']
		#
		#
		s = mdb.models['Model-1'].ConstrainedSketch(name='__profile2__', 
			sheetSize=4000.0)
		g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
		s.setPrimaryObject(option=STANDALONE)
		s.Spot(point=(0.0, -800.0))
		s.FixedConstraint(entity=v[0])
		s.Spot(point=(3604.0, -800.0))
		s.FixedConstraint(entity=v[1])
		s.Arc3Points(point1=(0.0, -800.0), point2=(3604.0, -800.0), point3=(1802.0, -455.0))
		s.VerticalDimension(vertex1=v[0], vertex2=v[4], textPoint=(-595.455078125, 
			-131.898254394531), value=360)
		s.HorizontalDimension(vertex1=v[4], vertex2=v[1], textPoint=(2994.951171875, 
			740.461364746094), value=1802.0)
		p = mdb.models['Model-1'].Part(name='as2', dimensionality=THREE_D, 
			type=DEFORMABLE_BODY)
		p = mdb.models['Model-1'].parts['as2']
		p.BaseWire(sketch=s)
		s.unsetPrimaryObject()
		p = mdb.models['Model-1'].parts['as2']
		del mdb.models['Model-1'].sketches['__profile2__']
		#-----------------------------------------------------------------------------
		#------------------------Assembly Module--------------------------------------
		#-----------------------------------------------------------------------------
		a = mdb.models['Model-1'].rootAssembly
		a.DatumCsysByDefault(CARTESIAN)
		p = mdb.models['Model-1'].parts['as1']
		a.Instance(name='as1', part=p, dependent=OFF)
		a = mdb.models['Model-1'].rootAssembly
		p = mdb.models['Model-1'].parts['as2']
		a.Instance(name='as2', part=p, dependent=OFF)
		#
		#
		#
		#
		#
		#-----------------------------------------------------------------------------
		#------------------------------STEP MODULE------------------------------------
		#-----------------------------------------------------------------------------
		#
		#Creating steps for the analysis, based on the number of required steps from the input file
		#
		#Decides first step based on input parameters
		firstStep='winching'
		if vars.makeBefore==True:
			firstStep='before'
		#Creating the first step first, so the default F- and H-output are created from the first step
		if vars.makeBefore==True:
			mdb.models['Model-1'].ExplicitDynamicsStep(name='before', previous='Initial', 
					timePeriod=float(vars.Tb[it]))
			mdb.models['Model-1'].ExplicitDynamicsStep(name='winching', previous='before', 
					timePeriod=float(vars.T[it]))
		else:
			mdb.models['Model-1'].ExplicitDynamicsStep(name='winching', previous='Initial', 
					timePeriod=float(vars.T[it]))
		if vars.makeAfter==True:
			mdb.models['Model-1'].ExplicitDynamicsStep(name='after', previous='winching', 
					timePeriod=float(vars.Ta[it]))
		#
		if vars.requestedFrames>20: #and it==vars.nIter-1:
			mdb.models['Model-1'].FieldOutputRequest(name='Model-1',createStepName=firstStep,numIntervals=vars.requestedFrames)
		#
		#
		#
		#-----------------------------------------------------------------------------
		#--------------------------Interaction Module---------------------------------
		#-----------------------------------------------------------------------------
		#
		#
		#------------------------------Interaction PROPERTIES-----------------------------------
		#SLIPRING DEFINITION
		mdb.models['Model-1'].ConnectorSection(name='slipring', assembledType=SLIPRING, 
		massPerLength=10.0)
		if vars.wire=='rigid':
			elastic_0 = connectorBehavior.ConnectorElasticity(components=(1, ), 
				behavior=RIGID)
		elif vars.wire=='elastic':
			elastic_0 = connectorBehavior.ConnectorElasticity(components=(1, ),
				table=((vars.EA[it], ), ))
		elastic_0.ConnectorOptions()
		friction_1 = connectorBehavior.ConnectorFriction(frictionModel=PREDEFINED)
		friction_1.TangentialBehavior(table=((float(vars.Mu[it]), ), ))
		friction_1.ConnectorOptions()
		mdb.models['Model-1'].sections['slipring'].setValues(behaviorOptions =(
		elastic_0, friction_1, ) )
		#
		#
		#
		a = mdb.models['Model-1'].rootAssembly
		r=a.referencePoints
		e1=a.instances['as1'].edges
		e2=a.instances['as2'].edges
		v1 = a.instances['as1'].vertices
		v2 = a.instances['as2'].vertices
		#
		p=mdb.models['Model-1'].parts['as1']
		L=p.getLength(p.edges)
		index=list()
		cuts=vars.cutAt(int(vars.nP[it]),vars.Lmid[it],vars.Lquart[it],vars.cutMethod)
		for i in range(int(vars.nP[it])):
			print(i)
			cutAt=float(cuts[i])
			a.PartitionEdgeByParam(edges=e1[0],parameter=cutAt)
			a.PartitionEdgeByParam(edges=e2[0],parameter=cutAt)
			index.append(i)
		for i in range(int(vars.nP[it])+1):
			if i>0 and i<vars.nP[it]:
				wire = a.WirePolyLine(points=((v1[i], v2[i+1]),
				    ), mergeType=IMPRINT, 
				meshable=False)
				oldName = wire.name
				mdb.models['Model-1'].rootAssembly.features.changeKey(fromName=oldName, 
				toName='slipRing_Line_1-'+str(1+i))
				e1 = a.edges
				edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
				a.Set(edges=edges1, name='slipRing_Line_1-'+str(1+i))
				region = mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_1-'+str(1+i)]
				csa = a.SectionAssignment(sectionName='slipring', region=region)
				#: The section "slipring" has been assigned to 1 wire or attachment line.
				#
				wire = a.WirePolyLine(points=((v2[i], v1[i+1]),
				    ), mergeType=IMPRINT, 
				meshable=False)
				oldName = wire.name
				mdb.models['Model-1'].rootAssembly.features.changeKey(fromName=oldName, 
				toName='slipRing_Line_2-'+str(1+i))
				e1 = a.edges
				edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
				a.Set(edges=edges1, name='slipRing_Line_2-'+str(1+i))
				region = mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_2-'+str(1+i)]
				csa = a.SectionAssignment(sectionName='slipring', region=region)
			#: The section "slipring" has been assigned to 1 wire or attachment line.
			#
		id=list()
		a.ReferencePoint(point=(0, -vars.Lw[it], 0))
		id.append(a.features['RP-1'].id)
		a.ReferencePoint(point=(0, -800+vars.Lw[it], 0))
		id.append(a.features['RP-2'].id)
		a.ReferencePoint(point=(3604, -vars.Lw[it], 0))
		id.append(a.features['RP-3'].id)
		a.ReferencePoint(point=(3604, -800+vars.Lw[it], 0))
		id.append(a.features['RP-4'].id)
		#
		#
		wire = a.WirePolyLine(points=((r[id[0]], v1[1]), ),
		    mergeType=IMPRINT, 
		meshable=False)
		oldName = wire.name
		mdb.models['Model-1'].rootAssembly.features.changeKey(fromName=oldName, 
		toName='slipRing_Line_1-start')
		e1 = a.edges
		edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
		a.Set(edges=edges1, name='slipRing_Line_1-start')
		region = mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_1-start']
		csa = a.SectionAssignment(sectionName='slipring', region=region)
		#
		wire = a.WirePolyLine(points=((r[id[1]], v2[1]), ),
		    mergeType=IMPRINT, 
		meshable=False)
		oldName = wire.name
		mdb.models['Model-1'].rootAssembly.features.changeKey(fromName=oldName, 
		toName='slipRing_Line_2-start')
		e1 = a.edges
		edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
		a.Set(edges=edges1, name='slipRing_Line_2-start')
		region = mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_2-start']
		csa = a.SectionAssignment(sectionName='slipring', region=region)
		#
		wire = a.WirePolyLine(points=((v1[int(vars.nP[it])],r[id[2]]),
		    ), mergeType=IMPRINT, 
		meshable=False)
		oldName = wire.name
		mdb.models['Model-1'].rootAssembly.features.changeKey(fromName=oldName, 
		toName='slipRing_Line_1-end')
		e1 = a.edges
		edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
		a.Set(edges=edges1, name='slipRing_Line_1-end')
		region = mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_1-end']
		csa = a.SectionAssignment(sectionName='slipring', region=region)
		#
		wire = a.WirePolyLine(points=((v2[int(vars.nP[it])],r[id[3]]),
		    ), mergeType=IMPRINT, 
		meshable=False)
		oldName = wire.name
		mdb.models['Model-1'].rootAssembly.features.changeKey(fromName=oldName, 
		toName='slipRing_Line_2-end')
		e1 = a.edges
		edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
		a.Set(edges=edges1, name='slipRing_Line_2-end')
		region = mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_2-end']
		csa = a.SectionAssignment(sectionName='slipring', region=region)
		#
		indexV=int(ceil(float(vars.nP[it])/2))
		v=v1[indexV:indexV+1]
		region=regionToolset.Region(vertices=v)
		region=a.Set(vertices=v,name='midPoint')
		#
		indexV=int(ceil(float(vars.nP[it])/4))
		v=v1[indexV:indexV+1]
		region=regionToolset.Region(vertices=v)
		region=a.Set(vertices=v,name='quarterPoint')
		#
		#-----------------------------------------------------------------------------
		#--------------------------Property MODULE------------------------------------
		#-----------------------------------------------------------------------------
		#--------Cross section geometry--------
		#
		#Defining a single pipe cross section depending on the spacing parameter given.
		D=0.9144
		t=0.0305
		R=D/2
		r_old=R-t
		L_const = (  (2 * R + vars.S[it]) / np.sqrt(3)  )**2
		L_var = np.cos(0)**2 + np.cos(120*np.pi/180)**2 + np.cos(240*np.pi/180)**2
		L_tot = ( L_const * L_var)
		A = np.pi * (R**2-r_old**2)
		I = (np.pi/4)*(R**4 - r_old**4)*3 + L_tot * A	#Not 3A since L_tot consists of three sums.
		r_mod=np.sqrt( (2*I)/(3*A) - (3*A)/(2*np.pi) )
		R_mod=np.sqrt((3*A) /np.pi + r_mod**2)
		I_new=np.pi/4*(R_mod**4-r_mod**4)
		A_new=np.pi*(R_mod**2-r_mod**2)
		t_mod=R_mod-r_mod
		#
		#The above block defines the cross sectional data to fit a given spacing S
		#The below block implements a one-pipe cross section in the model
		mdb.models['Model-1'].Material(name='steel')
		mdb.models['Model-1'].materials['steel'].Density(table=((7810.0, ), ))
		mdb.models['Model-1'].materials['steel'].Elastic(table=((207000000000.0, 0.3), ))
		mdb.models['Model-1'].materials['steel'].Damping(alpha=vars.D[it])
		mdb.models['Model-1'].PipeProfile(name='onePipe', r=R_mod, t=t_mod)
		mdb.models['Model-1'].BeamSection(name='beamSection', 
			integration=DURING_ANALYSIS, poissonRatio=0.3, profile='onePipe', 
			material='steel', temperatureVar=LINEAR, consistentMassMatrix=False)
		p = mdb.models['Model-1'].parts['as1']
		e = p.edges
		edges = e.getSequenceFromMask(mask=('[#1 ]', ), )
		region = regionToolset.Region(edges=edges)
		p.SectionAssignment(region=region, sectionName='beamSection', offset=0.0, 
			offsetType=MIDDLE_SURFACE, offsetField='', 
			thicknessAssignment=FROM_SECTION)
		p = mdb.models['Model-1'].parts['as2']
		e = p.edges
		edges = e.getSequenceFromMask(mask=('[#1 ]', ), )
		region = regionToolset.Region(edges=edges)
		p.SectionAssignment(region=region, sectionName='beamSection', offset=0.0, 
			offsetType=MIDDLE_SURFACE, offsetField='', 
			thicknessAssignment=FROM_SECTION)
		p = mdb.models['Model-1'].parts['as1']
		e = p.edges
		edges = e.getSequenceFromMask(mask=('[#1 ]', ), )
		region=regionToolset.Region(edges=edges)
		p = mdb.models['Model-1'].parts['as1']
		p.assignBeamSectionOrientation(region=region, method=N1_COSINES, n1=(0.0, 0.0, 
			-1.0))
		#: Beam orientations have been assigned to the selected regions.
		p = mdb.models['Model-1'].parts['as2']
		e = p.edges
		edges = e.getSequenceFromMask(mask=('[#1 ]', ), )
		region=regionToolset.Region(edges=edges)
		p = mdb.models['Model-1'].parts['as2']
		p.assignBeamSectionOrientation(region=region, method=N1_COSINES, n1=(0.0, 0.0, 
			-1.0))
		#: Beam orientations have been assigned to the selected regions.
			
		#
		#
		#---------------------------------------------------------------------------------
		#----------------------------------LOAD MODULE------------------------------------
		#---------------------------------------------------------------------------------
		#
		#Defining smoothstep amplitude for winching process
		mdb.models['Model-1'].SmoothStepAmplitude(name='smootherstep', timeSpan=STEP, 
					data=((0.0, 0.0), (vars.t[it], 1.0)))
		#
		#
		#Holding winches
		lockPoints=(r[id[0]],r[id[1]],r[id[2]],r[id[3]])
		region = regionToolset.Region(referencePoints=lockPoints)
		mdb.models['Model-1'].DisplacementBC(name='holdWinches', 
			createStepName='Initial', region=region, u1=SET, u2=SET, u3=SET,
			amplitude=UNSET, distributionType=UNIFORM, fieldName='', 
			localCsys=None)
		a = mdb.models['Model-1'].rootAssembly
		#
		#
		#Holding chosen degrees of freedom at wall
		v1 = a.instances['as1'].vertices
		v2 = a.instances['as2'].vertices
		v=v1[len(v1)-1:len(v1)]+v1[0:1]+v2[len(v2)-1:len(v2)]+v2[0:1] 
		region = regionToolset.Region(vertices=v)
		region = a.Set(vertices=v, name='wallNodes')
		if vars.doLockAll==True:
			mdb.models['Model-1'].DisplacementBC(name='holdAtWalls', 
				createStepName='Initial', region=region, u1=SET, u2=SET, u3=SET, ur1=SET, ur2=SET, ur3=SET, amplitude=UNSET, distributionType=UNIFORM, fieldName='', 
				localCsys=None)
		elif vars.doLockAll==False:
			mdb.models['Model-1'].DisplacementBC(name='holdAtWalls', 
				createStepName='Initial', region=region, u1=SET, u2=SET, u3=SET, amplitude=UNSET, distributionType=UNIFORM, fieldName='', 
				localCsys=None)
		#
		#Defining material flow in winches
		a = mdb.models['Model-1'].rootAssembly
		r = a.referencePoints
		refPoints1=(r[id[0]],r[id[1]], )
		region = regionToolset.Region(referencePoints=refPoints1)
		mdb.models['Model-1'].MaterialFlowBC(name='matFlowInnerWinch', createStepName='Initial', 
			region=region, distributionType=UNIFORM, fieldName='', magnitude=0.0)
		mdb.models['Model-1'].boundaryConditions['matFlowInnerWinch'].setValuesInStep(
			stepName='winching', magnitude=-vars.dL[it]*0.5, amplitude='smootherstep')
		if vars.makeAfter==True:
			mdb.models['Model-1'].boundaryConditions['matFlowInnerWinch'].setValuesInStep(
				stepName='after', magnitude=0.0, amplitude=FREED)
		#		
		refPoints1=(r[id[2]],r[id[3]], )
		region = regionToolset.Region(referencePoints=refPoints1)
		mdb.models['Model-1'].MaterialFlowBC(name='matFlowOuterWinch', createStepName='Initial', 
			region=region, distributionType=UNIFORM, fieldName='', magnitude=0.0)
		mdb.models['Model-1'].boundaryConditions['matFlowOuterWinch'].setValuesInStep(
			stepName='winching', magnitude=vars.dL[it]*0.5, amplitude='smootherstep')
		if vars.makeAfter==True:
			mdb.models['Model-1'].boundaryConditions['matFlowOuterWinch'].setValuesInStep(
				stepName='after', magnitude=0.0, amplitude=FREED)
		#
		#
		#Apply load for water currents
		if vars.Fb[it]==True or vars.Fa[it]==True:
			if vars.Fb[it]==True:
				loadStep='before'
			elif vars.Fa[it]==True:
				loadStep='after'
			a = mdb.models['Model-1'].rootAssembly
			e1 = a.instances['as1'].edges
			edges1 = e1.getSequenceFromMask(mask=('[#3ffff ]', ), )
			e2 = a.instances['as2'].edges
			edges2 = e2.getSequenceFromMask(mask=('[#3ffff ]', ), )
			region = a.Set(edges=edges1+edges2, name='bothBundles')
			mdb.models['Model-1'].LineLoad(name='waterCurrent', createStepName=loadStep, 
				region=region, comp2=425.0)
		#
		#
		#---------------------------------------------------------------------------------
		#-----------------------------------MESH MODULE------------------------------------
		#---------------------------------------------------------------------------------
		#
		a = mdb.models['Model-1'].rootAssembly
		partInstances =(a.instances['as1'], )
		a.seedPartInstance(regions=partInstances, size=vars.Le[it], deviationFactor=0.1, 
			minSizeFactor=0.1)
		elemType1 = mesh.ElemType(elemCode=B31, elemLibrary=EXPLICIT)
		e1 = a.instances['as1'].edges
		edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
		pickedRegions =(edges1, )
		a.setElementType(regions=pickedRegions, elemTypes=(elemType1, ))
		partInstances =(a.instances['as1'], )
		a.generateMesh(regions=partInstances)
		partInstances =(a.instances['as2'], )
		a.seedPartInstance(regions=partInstances, size=vars.Le[it], deviationFactor=0.1, 
			minSizeFactor=0.1)
		elemType1 = mesh.ElemType(elemCode=B31, elemLibrary=EXPLICIT)
		e1 = a.instances['as2'].edges
		edges1 = e1.getSequenceFromMask(mask=('[#1 ]', ), )
		pickedRegions =(edges1, )
		a.setElementType(regions=pickedRegions, elemTypes=(elemType1, ))
		partInstances =(a.instances['as2'], )
		a.generateMesh(regions=partInstances)
		#
		#
		#---------------------------------------------------------------------------------
		#-----------------------------------HISTORY OUTPUT--------------------------------
		#---------------------------------------------------------------------------------
		#
		#Using a variable, so the H-output are created from the first step
		a = mdb.models['Model-1'].rootAssembly
		regionDef=mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_1-start']
		mdb.models['Model-1'].HistoryOutputRequest(name='winchForce_start', 
			createStepName=firstStep, variables=('CTF1','CP1','CP2','CP3' ), region=regionDef, 
			sectionPoints=DEFAULT, rebar=EXCLUDE,timeInterval=vars.extractDataAtTimes[it])
		#
		regionDef=mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_2-'+str(int((vars.nP[it]+1)/2))]
		mdb.models['Model-1'].HistoryOutputRequest(name='winchForce_mid', 
			createStepName=firstStep, variables=('CTF1', 'CP1','CP2','CP3'), region=regionDef, 
			sectionPoints=DEFAULT, rebar=EXCLUDE,timeInterval=vars.extractDataAtTimes[it])
		#
		a = mdb.models['Model-1'].rootAssembly
		regionDef=mdb.models['Model-1'].rootAssembly.sets['slipRing_Line_1-end']
		mdb.models['Model-1'].HistoryOutputRequest(name='winchForce_end', 
			createStepName=firstStep, variables=('CTF1', 'CP1','CP2','CP3'), region=regionDef, 
			sectionPoints=DEFAULT, rebar=EXCLUDE,timeInterval=vars.extractDataAtTimes[it])
		#
		#
		a = mdb.models['Model-1'].rootAssembly
		regionDef=mdb.models['Model-1'].rootAssembly.sets['midPoint']
		mdb.models['Model-1'].HistoryOutputRequest(name='midPointPosition', 
			createStepName=firstStep, variables=('U2',), region=regionDef, 
			sectionPoints=DEFAULT, rebar=EXCLUDE,timeInterval=vars.extractDataAtTimes[it])
		#
		#
		nElementsOnEdge=len(a.instances['as1'].elements)
		expressions=list()
		for i in range(nElementsOnEdge):
			innerExpression=(i+1,1,1,1)
			expressions.append(innerExpression)
		session.Path(name='upperBundle', type=5, expression=(('AS1',(expressions)),))
		#
		a = mdb.models['Model-1'].rootAssembly
		e1 = a.edges
		#edges1 = e1.getSequenceFromMask(mask=('[#999999a ]', ), )
		edges1 = e1.getSequenceFromMask(mask=('[#9999999a #99 ]', ), )
		a.Set(edges=edges1, name='winchLine_upperStart')
		#: The set 'winchLine_2' has been created (14 edges).
		session.viewports['Viewport: 1'].assemblyDisplay.setValues(
			adaptiveMeshConstraints=ON)
		regionDef=mdb.models['Model-1'].rootAssembly.sets['winchLine_upperStart']
		mdb.models['Model-1'].FieldOutputRequest(name='winchLine_upperStart', 
			createStepName=firstStep, variables=('CTF','CU'), 
			region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE, numIntervals=vars.requestedFrames)
		#
		#
		#
		a = mdb.models['Model-1'].rootAssembly
		e1 = a.edges
		#edges1 = e1.getSequenceFromMask(mask=('[#6666665 ]', ), )
		edges1 = e1.getSequenceFromMask(mask=('[#66666665 #66 ]', ), )
		a.Set(edges=edges1, name='winchLine_lowerStart')
		session.viewports['Viewport: 1'].assemblyDisplay.setValues(
			adaptiveMeshConstraints=ON)
		regionDef=mdb.models['Model-1'].rootAssembly.sets['winchLine_lowerStart']
		mdb.models['Model-1'].FieldOutputRequest(name='winchLine_lowerStart', 
			createStepName=firstStep, variables=('CTF', 'CU'), 
			region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE, numIntervals=vars.requestedFrames)
		#
		#Defining all nodes on upper bundle in a set.(Not yet in use, left in case of future convenience)
		#n1 = a.instances['as1'].nodes
		#nodes1 = n1.getSequenceFromMask(mask=('[#ffffffff:2 #7f ]', ), )
		#a.Set(nodes=nodes1, name='upperBundleNodes')
		#
		#
		#Same with all the elements
		e1 = a.instances['as1'].elements
		elements1 = e1.getSequenceFromMask(mask=('[#ffffffff:2 ]', ), )
		#elements1 = e1.getSequenceFromMask(mask=('[#ffffffff:2 #3fff ]', ), )
		a.Set(elements=e1, name='upperBundleElements')
		regionDef=mdb.models['Model-1'].rootAssembly.sets['upperBundleElements']
		mdb.models['Model-1'].HistoryOutputRequest(name='elementOutputUpperBundle', 
			createStepName=firstStep, variables=('NFORC',), region=regionDef, sectionPoints=DEFAULT, 
			rebar=EXCLUDE)
		#---------------------------------------------------------------------------------
		#-----------------------------------JOB MODULE------------------------------------
		#---------------------------------------------------------------------------------
		#
		name1='as_scripted_'+str(count)
		mdb.Job(name=name1, model='Model-1', description='', type=ANALYSIS, 
					atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90, 
					memoryUnits=PERCENTAGE, explicitPrecision=SINGLE, 
					nodalOutputPrecision=SINGLE, echoPrint=OFF, modelPrint=OFF, 
					contactPrint=OFF, historyPrint=OFF, userSubroutine='', scratch='', 
					parallelizationMethodExplicit=DOMAIN, numDomains=1, 
					activateLoadBalancing=False, multiprocessingMode=DEFAULT, numCpus=1)
		mdb.jobs[name1].submit(consistencyChecking=OFF)
		mdb.jobs[name1].waitForCompletion()
		mdb.saveAs(pathName='as_scripted')
		#
		#
		#---------------------------------------------------------------------------------
		#--------------------------------EXTRACTING DATA----------------------------------
		#---------------------------------------------------------------------------------
		#
		#
		#: The model database 'C:/Temp/master/bjelkemodell/0903_fullScript/as_scripted'
		#Assigning the name of the last step to a variable
		lastStep='winching'
		if vars.makeAfter==True:
			lastStep='after'
		
		o3 = session.openOdb(
			name='as_scripted_'+str(count)+'.odb')
		odb = session.odbs['as_scripted_'+str(count)+'.odb']
		#
		odb = session.odbs['as_scripted_'+str(count)+'.odb']
		session.viewports['Viewport: 1'].setValues(displayedObject=odb)
		session.viewports['Viewport: 1'].view.setValues(session.views['Front'])
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			connectorDisplay=ON, highlightConnectorPts=OFF, showConnectorAxes=OFF, 
			showConnectorType=OFF)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			beamScaleFactor=10)
		session.printOptions.setValues(vpBackground=OFF,vpDecorations=OFF, reduceColors=True)
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(triad=OFF,state=OFF,title=OFF,legend=OFF)
		session.viewports['Viewport: 1'].view.setValues(nearPlane=20000,farPlane=20001, width=10000, height=800, viewOffsetX=0, 
			viewOffsetY=0)
		#session.printOptions.setValues(vpDecorations=OFF, vpBackground=OFF)
		session.viewports[session.currentViewportName].odbDisplay.setFrame(
			step=lastStep, frame=10000000)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			renderBeamProfiles=ON)
		session.epsOptions.setValues(resolution=DPI_600,imageSize=(5.20, 2.41))
		session.printToFile(
			fileName='../plots/overview_'+str(count), 
			format=PNG, canvasObjects=(session.viewports['Viewport: 1'], ))
		session.printToFile(
			fileName='../plots/overview_'+str(count), 
			format=EPS, canvasObjects=(session.viewports['Viewport: 1'], ))

		#
		#-----------------------------Producing relevant data sets----------------------------------------------------------
		#
		nElset_1_START=int((vars.nP[it])*2-1)
		nElset_1_END=int((vars.nP[it])*2+1)
		nElset_2_MID=int(vars.nP[it]-1)
		#
		#Makeing a list with the active steps in this analysis, based on user input
		activeSteps=list()
		if vars.makeBefore==True:
			activeSteps.append('before')
		activeSteps.append('winching')
		if vars.makeAfter==True:
			activeSteps.append('after')
		#
		#
		session.XYDataFromHistory(name='winchForce_start', odb=odb, 
			outputVariableName='Connector element total force: CTF1 PI: rootAssembly Element '+str(nElset_1_START)+' in ELSET SLIPRING_LINE_1-START', 
			steps=activeSteps, )
		#
		#
		session.XYDataFromHistory(name='winchForce_end', odb=odb, 
			outputVariableName='Connector element total force: CTF1 PI: rootAssembly Element '+str(nElset_1_END)+' in ELSET SLIPRING_LINE_1-END', 
			steps=activeSteps, )
		#
		#
		session.XYDataFromHistory(name='winchForce_mid', odb=odb, 
			outputVariableName='Connector element total force: CTF1 PI: rootAssembly Element '+str(nElset_2_MID)+' in ELSET SLIPRING_LINE_2-'+str(int((vars.nP[it]+1)/2)), 
			steps=activeSteps, )
		#
		#
		session.XYDataFromHistory(name='U2_midPoint', odb=odb, 
			outputVariableName='Spatial displacement: U2 PI: AS1 Node '+str(int((vars.nP[it]+1)/2+1))+' in NSET MIDPOINT', 
			steps=activeSteps, )
		#
		#
		session.XYDataFromHistory(name='CP2_start', odb=odb, 
			outputVariableName='Connector element relative position: CP3 PI: rootAssembly Element '+str(nElset_1_START)+' in ELSET SLIPRING_LINE_1-START', 
			steps=activeSteps, )
		#
		#
		session.XYDataFromHistory(name='CP2_mid', odb=odb, 
			outputVariableName='Connector element relative position: CP2 PI: rootAssembly Element '+str(nElset_2_MID)+' in ELSET SLIPRING_LINE_2-'+str(int((vars.nP[it]+1)/2)), 
			steps=activeSteps, )
		#
		#
		session.XYDataFromHistory(name='CP2_end', odb=odb, 
			outputVariableName='Connector element relative position: CP2 PI: rootAssembly Element '+str(nElset_1_END)+' in ELSET SLIPRING_LINE_1-END', 
			steps=activeSteps, )
		#
		#
		#
		session.viewports['Viewport: 1'].setValues(displayedObject=o3)
		session.viewports['Viewport: 1'].odbDisplay.setPrimaryVariable(
			variableLabel='U', outputPosition=NODAL, refinement=(COMPONENT, 
			'U2'))
		pth = session.paths['upperBundle']
		session.XYDataFromPath(name='U2_along_path', path=pth, 
		includeIntersections=False, pathStyle=PATH_POINTS, numIntervals=1, 
		shape=DEFORMED, labelType=TRUE_DISTANCE)
		#
		#
		session.viewports['Viewport: 1'].view.setValues(session.views['Front'])
		session.viewports['Viewport: 1'].odbDisplay.display.setValues(plotState=(
			CONTOURS_ON_DEF, ))
		session.viewports['Viewport: 1'].odbDisplay.setPrimaryVariable(
			variableLabel='CTF', outputPosition=WHOLE_ELEMENT, refinement=(COMPONENT, 
			'CTF1'), )
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			connectorDisplay=ON, highlightConnectorPts=OFF, showConnectorAxes=OFF, 
			showConnectorType=OFF)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			averagingThreshold=0)
		#
		#Below is some new graphic testing
		#
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(
			legendNumberFormat=ENGINEERING)
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(
		legendPosition=(1, 73))
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(
			legendFont='-*-verdana-medium-r-normal-*-*-180-*-*-p-*-*-*')
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(legend=ON, legendBox=OFF, triad=OFF,state=OFF,title=OFF)
		session.viewports['Viewport: 1'].view.setValues(nearPlane=20000,farPlane=20001, width=6600, height=800, viewOffsetX=-3300, 
			viewOffsetY=150)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			renderBeamProfiles=OFF,noResultsColor='Black')
		session.printOptions.setValues(vpDecorations=OFF, vpBackground=OFF)
		session.epsOptions.setValues(resolution=DPI_300)
		session.printToFile(
			fileName='../plots/wireForcePlotEPS_'+str(int(it)), 
			format=EPS, canvasObjects=(session.viewports['Viewport: 1'], ))
		session.printToFile(
			fileName='../plots/wireForcePlotPNG_'+str(int(it)), 
			format=PNG, canvasObjects=(session.viewports['Viewport: 1'], ))
			#
			#
			#
			#
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			sectionResults=USE_ENVELOPE, envelopeDataPosition=ELEMENT_NODAL)
		session.viewports['Viewport: 1'].odbDisplay.setPrimaryVariable(
			variableLabel='S', outputPosition=INTEGRATION_POINT, refinement=(COMPONENT, 
			'S11'))
		session.viewports['Viewport: 1'].odbDisplay.display.setValues(plotState=(
		CONTOURS_ON_DEF, ))
		session.viewports['Viewport: 1'].odbDisplay.contourOptions.setValues(
			tickmarkPlots=OFF)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			connectorDisplay=ON, highlightConnectorPts=OFF, showConnectorAxes=OFF, 
			showConnectorType=OFF, noResultsColor='Gray')
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(
			legendNumberFormat=ENGINEERING)
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(
		legendPosition=(1, 73))
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(
			legendFont='-*-verdana-medium-r-normal-*-*-180-*-*-p-*-*-*', )
		session.viewports['Viewport: 1'].viewportAnnotationOptions.setValues(legend=ON, legendBox=OFF, triad=OFF, state=OFF,title=OFF)
		session.viewports['Viewport: 1'].view.setValues(nearPlane=20000,farPlane=20001,width=6800, height=800, viewOffsetX=-3400, 
			viewOffsetY=200)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			renderBeamProfiles=ON)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			beamScaleFactor=10)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			averagingThreshold=0)
		session.epsOptions.setValues(resolution=DPI_300)
		session.printToFile(
			fileName='../plots/S11_seabed_'+str(int(it)), 
			format=PNG, canvasObjects=(session.viewports['Viewport: 1'], ))
		session.printToFile(
			fileName='../plots/S11_seabed_'+str(int(it)), 
			format=EPS, canvasObjects=(session.viewports['Viewport: 1'], ))
		#
		#
		odb = session.odbs['as_scripted_'+str(count)+'.odb']
		elementsInUpperBundle = mdb.models['Model-1'].rootAssembly.sets['upperBundleElements'].elements
		k=0
		nElems=len(elementsInUpperBundle)
		for el in elementsInUpperBundle: 
			k+=1
			element_label = el.label  #integer value 
			element_nodes = el.connectivity #tuple of integers containing node labels defining the elements  
			session.XYDataFromHistory(name='NFORC1_'+str(k),odb=odb, 
				outputVariableName='Forces at element nodes caused by stress in the element: NFORC1 PI: AS1 Element '+str(element_label)+' Node '+str(element_nodes[0]+1)+' in ELSET UPPERBUNDLEELEMENTS', 
				steps=activeSteps, )
			#if k-1=nElems:
			#	session.XYDataFromHistory(name='NFORC1_'+str(k),odb=odb, 
			#	outputVariableName='Forces at element nodes caused by stress in the element: NFORC1 PI: AS1 Element '+str(element_label)+' Node '+str(element_nodes[0]+1)+' in ELSET UPPERBUNDLEELEMENTS', 
			#	steps=('winching', ), )
		session.viewports['Viewport: 1'].setValues(displayedObject=o3)
		session.viewports['Viewport: 1'].odbDisplay.setPrimaryVariable(
			variableLabel='S', outputPosition=INTEGRATION_POINT, refinement=(COMPONENT, 
			'S11'))
		# session.viewports['Viewport: 1'].odbDisplay.setPrimarySectionPoint(
		# sectionPoint={'beam < pipe > < elset = ASSEMBLY_AS1__PICKEDSET2 >':(
		# 'Angle = 180.0000, (1-fraction = -1.000000, 2-fraction = 0.000000)', 
		# 'Angle = 0.0000, (1-fraction = 1.000000, 2-fraction = 0.000000)')})
		session.viewports[session.currentViewportName].odbDisplay.setFrame(
			step=lastStep, frame=10000000)
		session.viewports['Viewport: 1'].odbDisplay.setPrimaryVariable(
			variableLabel='S', outputPosition=INTEGRATION_POINT, refinement=(COMPONENT, 
			'S11'))
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			sectionResults=USE_ENVELOPE, envelopeDataPosition=CENTROID, envelopeCriteria=MAX_ABS_VALUE)
		session.viewports['Viewport: 1'].odbDisplay.basicOptions.setValues(
			envelopeCriteria=MAX_ABS_VALUE)
		pth = session.paths['upperBundle']
		session.XYDataFromPath(name='S11_along_path', path=pth, 
			includeIntersections=False, pathStyle=PATH_POINTS, numIntervals=1, 
			shape=DEFORMED, labelType=TRUE_DISTANCE)
		#
		#
		#----------------------------Saving data to .txt-files--------------------------------------------------------------
		#
		x0 = session.xyDataObjects['winchForce_start']
		session.writeXYReport(fileName='../XYdata/CTF1_start_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['winchForce_end']
		session.writeXYReport(fileName='../XYdata/CTF1_end_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['winchForce_mid']
		session.writeXYReport(fileName='../XYdata/CTF1_mid_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['U2_midPoint']
		session.writeXYReport(fileName='../XYdata/U2_midPoint_'+str(count)+'.txt', xyData=(x0, ))
		#
		#x0 = session.xyDataObjects['U2_quarterPoint']
		#session.writeXYReport(fileName='odb-data/U2_quarterPoint_'+vars.id+'_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['CP2_start']
		session.writeXYReport(fileName='../XYdata/CP2_start_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['CP2_mid']
		session.writeXYReport(fileName='../XYdata/CP2_mid_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['CP2_end']
		session.writeXYReport(fileName='../XYdata/CP2_end_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['S11_along_path']
		session.writeXYReport(fileName='../XYdata/S11_path_'+str(count)+'.txt', xyData=(x0, ))
		#
		x0 = session.xyDataObjects['U2_along_path']
		session.writeXYReport(fileName='../XYdata/U2_path_'+str(count)+'.txt', xyData=(x0, ))
		#
		k=0
		for el in elementsInUpperBundle: 
			k+=1
			x0 = session.xyDataObjects['NFORC1_'+str(k)]
			session.writeXYReport(fileName='../XYdata/NFORC_'+str(k)+'_'+str(count)+'.txt', xyData=(x0, ))
			nElements=k
		efile=open('../xydata/nElements_'+str(int(it))+'.txt','w')
		efile.write(str(nElements))
		
		#
		#
		timeSpent=timer.time()-startTime
		if vars.useGUI==False:
			file.write('Time spent on iteration number: '+str(int(count+1))+' was '+str(timeSpent)+' seconds.\r\n')
		#
		count=count+1
		#
		#Iteration loop ending---------------------------------------------------------------------
	if vars.useGUI==False:
		timeSpentTotal=timer.time()-startTimeTotal
		file.write('Total time all analyses: '+str(timeSpentTotal)+' seconds.\r\n')
		file.close()
		#
		#

